function cobra(outfilebase,anat_file,angle_file,ecc_file, ...
               stim_freq,tf_ignore,directions,add_outputs, ...
               xfm_file,invxfm_file,var_file,varargin);
% COBRA - automatically identify the visual field signs (VFS)
% cobra(outfilebase,anat_file,angle_file,ecc_file,stim_freq...
%               [,tf_ignore[,directions[,add_outputs...
%               [,xfm_file[,invxfm_file[,var_file,[varargin]]]]]]]);
%
% Program to automatically identify the visual field signs (VFS).
%
% ----------------------------------------------------------------------
% Input : 
%         outfilebase  : output name base,
%         anat_file    : anatomy (either t1 or classified white matter only),
%         angle_file   : fMRI file(s) with polar-angle stimulation,
%         ecc_file     : fMRI file(s) with eccentricity stimulation,
%         stim_freq    : one by two matric with the stimulation
%                        frequency for the angle and ecc files
% Optional input:
%         tf_ignore    : initial time frames to ignore for the
%                        angle and ecc files [default = [0 0])
%         directions   : matrix with the direction of fMRI
%                        stimulation (1 or -1) for angle_file(s) (column 1)
%                        ecc_file(s) (column 2), the rows are the
%                        number of files (padd with ones if
%                        different number of angle_files and ecc_files) 
%                        [default = ones]
%         add_outputs  : logical matrix indicating which additional
%                        outputs indicated by 1 (automatically padded by zeros)
%                        [default = 0 0]: VFS, mVFS, and average magnitude maps,
%                         1: individual data for eccentricity and
%                            polar-angle,
%                         2: t statistical map(s) using spearman rank
%                            order statistics,
%                         3: % BOLD signal change maps.
%         xfm_file     : tranformation file from native to talairach,
%                        one or same number as angle_files
%                        [default = []],
%         invxfm_file  : inverse of previous one [default = []],
%         var_file     : file containing variables used; 
%                        edit according to your needs.
%                        [default = 'cobra_options']
%         varargin     : define optional analysis parameters
%                        see "cobra options?" for more information.  
%
% ----------------------------------------------------------------------
% Output: outfilebase_VFS.mnc  : volume with Visual Field Signs for each voxel,
%         outfilebase_mVFS.mnc : same as previous one weighted by the
%                                magnitude of the signal in each voxel,
%         outfilebase_avg_fft_magnitude.mnc
%		               : average magnitude of signal in polar angle
%                                and eccentricity data,
%
% ----------------------------------------------------------------------

% ---------------------------------
% Copyright (c) 2003-2004 Serge Dumoulin
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License version 2
% as published by the Free Software Foundation.
% ---------------------------------

%-----------------------------------------------------------------------
% program structure (main functions used):
%  -RetinoGlobals           (define parameters to be used)
%  -CreateTMPdir            (create unique tmp-dir)
%  -RetPhaseAnalysis        (get phase and magnitude data)
%    phase_fit
%    [fm_resample2          (if(FAST), use fm_resample2 to prevent artifacts)]
%  -PhaseShift              (shift phase data)
%  [getvisualquadrants      (get visual quadrants if requested)]       
%  -GetDerivatives2         (compute derivatives phase data sets)
%    RetAvg2                 
%  -RetAvg2                 (combine derivatives)
%  -RetAnatPreProcess       (get resolution matched derivatives anatomy file)
%  -calc_vfs                (calculate visual field signs)
%
% minc-functions called:
%  mincblur
%  mincmath
%  mincresample
%  mincaverage
%  (evt.) xfminvert
%-----------------------------------------------------------------------

%-----------------------------------------------------------------------
% known bugs: (to be fixed sometime)
%  -does not crash gracefully
%  -cannot start or stop halfway 
%   (copying and pasting into matlab will work though ---
%    making sure all the variables are there, ie load variables.mat
%    from the temporary directory)
%  -program could be sped up quite a lot,
%   however i'm not sure about the trade off's (so left it that way)
%-----------------------------------------------------------------------

%-----------------------------------------------------------------------
%--- input checks and path stuff
%-----------------------------------------------------------------------
if nargin < 5,
   if nargin == 1,
      if ~isempty(strmatch('opt',lower(outfilebase))),
         help('cobra_options');
         return;
      else,
         help(mfilename);
         return;
      end;
   else,
       help(mfilename);
       return;
   end;
else,
   %--- add absolute path to name
   outfilebase=makepathabs(outfilebase);
   anat_file  =makepathabs(anat_file);
   if ~exist(anat_file,'file'),
       disp(sprintf('Warning: cannot find %s',anat_file));
   end;
   
   new_angle_file=[];
   for n=1:size(angle_file,1),
      tmp=makepathabs(angle_file(n,:));
      if ~exist(tmp,'file'),
         disp(sprintf('Warning: cannot find %s',tmp));
      end;   
      new_angle_file=strvcat(new_angle_file,tmp);
   end;
   angle_file=new_angle_file;
   
   
   new_ecc_file=[];
   for n=1:size(ecc_file,1),
      tmp=makepathabs(ecc_file(n,:));
      if ~exist(tmp,'file'),
         disp(sprintf('Warning: cannot find %s',tmp));
      end;   
      new_ecc_file=strvcat(new_ecc_file,tmp);
   end;
   ecc_file=new_ecc_file;
   clear new_angle_file new_ecc_file tmp;   
end;
if nargin < 6 | isempty(tf_ignore),
  tf_ignore = [0 0];
end;
if nargin < 7 | isempty(directions), 
  directions = ones(max([size(angle_file,1) size(ecc_file,1)]),2);
end;
if nargin < 8 | isempty(add_outputs),
  add_outputs = zeros(3,1);
end;
if nargin < 9,
   xfm_file = [];
else,
   new_xfm_file=[];
   for n=1:size(xfm_file,1),
      tmp=makepathabs(xfm_file(n,:));
      if ~exist(tmp,'file'),
         disp(sprintf('Warning: cannot find %s',tmp));
      end;   
      new_xfm_file=strvcat(new_xfm_file,tmp);
   end;
   xfm_file=new_xfm_file;
   clear new_xfm_file tmp;
end;
if nargin < 10,
   invxfm_file=[];
else,
   new_invxfm_file=[];
   for n=1:size(invxfm_file,1),
      tmp=makepathabs(invxfm_file(n,:));
      if ~exist(tmp,'file'),
         disp(sprintf('Warning: cannot find %s',tmp));
      end;   
      new_invxfm_file=strvcat(new_invxfm_file,tmp);
   end;
   invxfm_file=new_invxfm_file;
   clear new_xfm_file tmp;
end;
if nargin < 11 | isempty(var_file),
   var_file='cobra_options';
end;

%--- number of input file checks
if size(angle_file,1)~=size(ecc_file,1),
   disp(sprintf(['Warning: not the same amount of ' ...
                 'polar-angle files (%d) and eccentricity files (%d)'], ...
                 size(angle_file,1),size(ecc_file,1)));
end;
if ~isempty(xfm_file),
   if size(angle_file,1)~=size(xfm_file,1) & size(xfm_file,1)~=1,
      disp(sprintf(['Error: unequal amount of ' ...
                    'polar-angle files (%d) and xfm_files (%d)'], ...
                    size(angle_file,1),size(xfm_file,1)));
      return;
   end;
   if ~isempty(invxfm_file),
      if size(xfm_file,1)~=size(invxfm_file,1),
         disp(sprintf(['Error: unequal amount of ' ...
                       'xfm files (%d) and inverse xfm_files (%d)'], ...
                       size(xfm_file,1),size(invxfm_file,1)));
         return;
      end;
   end;
end;

%--- allow for different stimulus freq. for polar-angle and eccentricity
if prod(size(stim_freq))==1,
   stim_freq = [stim_freq; stim_freq];
elseif prod(size(stim_freq)) > 2,
   disp(sprintf('Warning: too many variables (%d) for stim_freq',...
                prod(size(stim_freq))));
   return;
end;

%--- same for initial time frames to ignore
if prod(size(tf_ignore))==1,
   tf_ignore = [tf_ignore; tf_ignore];
elseif prod(size(tf_ignore)) > 2,
   disp(sprintf('Warning: too many variables (%d) for tf_ignore',...
                prod(size(tf_ignore))));
   return;
end;

%--- additional outputs check and zero-padding
if prod(size(add_outputs))<3,
  add_outputs= [add_outputs(:); ...
                zeros(3-prod(size(add_outputs)),1)];
elseif prod(size(add_outputs))>3,
  disp(sprintf('[%s]: Warning: more than 3 (%d) outputs requested',...
               mfilename,prod(size(add_outputs))));
end;

%--- print out input variables for confirmation
outfilebase
anat_file
angle_file
ecc_file
stim_freq
tf_ignore
directions
add_outputs
xfm_file
invxfm_file
var_file


%-----------------------------------------------------------------------
%--- load variables into one struct variable
%-----------------------------------------------------------------------

%--- timing
starttime=clock;
startcpu =cputime;

%--- reset errors
lasterr('');

%--- load variables
if ~isempty(varargin),
    [cobra_param]=feval(var_file,varargin);
else,
    [cobra_param]=feval(var_file);
end;
disp(sprintf('[%s]: using variables defined in: %s',mfilename,var_file)); 

%--- create tmp-dir
[cobra_param.ret_tmp_subdir]=CreateTMPdir(cobra_param.ret_tmp_dir,mfilename);
disp(sprintf('[%s]: using temporary directory: %s',...
             mfilename,cobra_param.ret_tmp_subdir)); 


%--- obsolete
if isempty(cobra_param.like_file),
   cobra_param.like_file = anat_file;
end;

%--- obsolete?
if isempty(cobra_param.derivatives_kernel),
   [dimSteps]=GetMincDims([angle_file;ecc_file  ]);
   % order the steps in descending order: large to small
   which_dimSteps = fliplr(unique(dimSteps)); 
   cobra_param.derivatives_kernel=which_dimSteps(:,1);
end;

%--- check dimensions
if prod(GetMincDims(angle_file(1,:))) <= prod(GetMincDims(anat_file)),
    disp(sprintf('Warning: dimensions of functional file is equal or smaller than anatomy'));
end;


%--- add given variables to cobra_param -> one big ugly struct variable
cobra_param.outfilebase= outfilebase;
cobra_param.anat_file  = anat_file;
cobra_param.angle_file = angle_file;
cobra_param.ecc_file   = ecc_file;
cobra_param.stim_freq  = stim_freq;
cobra_param.blanks     = tf_ignore;
cobra_param.directions = directions;
cobra_param.add_outputs= add_outputs;
cobra_param.xfm_file   = xfm_file;
cobra_param.invxfm_file= invxfm_file;
cobra_param.var_file   = var_file;


%--- save all variables in tmpdir
MySaveName = [cobra_param.ret_tmp_subdir ...
              sprintf('%s_variables.mat',mfilename)];
save(MySaveName);


%-----------------------------------------------------------------------
%--- start analysis
%-----------------------------------------------------------------------
cd(cobra_param.ret_tmp_subdir);

%-----------------------------------------------------------------------
%--- phase analysis files
%-----------------------------------------------------------------------
%--- polar angle phase map
cobra_phaseanalysis(cobra_param);

if cobra_param.add_outputs(1),
   copyfile(['phase1ecc____fft_phase.mnc'], ...
            [outfilebase '_ecc_fft_phase.mnc']);
   copyfile(['phase2angle__fft_phase.mnc'], ...
            [outfilebase '_pol_fft_phase.mnc']);  
   copyfile(['phase1ecc____fft_magnitude.mnc'], ...
            [outfilebase '_ecc_fft_magnitude.mnc']);
   copyfile(['phase2angle__fft_magnitude.mnc'], ...
            [outfilebase '_pol_fft_magnitude.mnc']);
end;

if cobra_param.add_outputs(2),
   % average the two t-maps together and 
   % roughly) correct for the averaging process
   system(['mincaverage phase1ecc____dspear_tmap.mnc ' ...
         'phase2angle__dspear_tmap.mnc ' ...
         'phase_avg_dspear_tmap.mnc ']);
   system(['mincmath -mult -const 1.4142 ' ...
         'phase_avg_dspear_tmap.mnc ' ...
         outfilebase '_avg_dspear_tmap.mnc']);
   %minmimum
%    system(['mincmath -min ' 'phase1ecc____dspear_tmap.mnc ' ...
%            'phase2angle__dspear_tmap.mnc ' ...
%            outfilebase '_min_dspear_tmap.mnc ']);
   if cobra_param.add_outputs(1),
      copyfile(['phase1ecc____dspear_tmap.mnc'], ...
               [outfilebase '_ecc_dspear_tmap.mnc']);
      copyfile(['phase2angle__dspear_tmap.mnc'], ...
               [outfilebase '_pol_dspear_tmap.mnc']);
   end;
end;

if cobra_param.add_outputs(3),
   system(['mincaverage phase1ecc____perc_bold_signal.mnc ' ...
           'phase2angle__perc_bold_signal.mnc '  ...
            outfilebase '_avg_perc_signal.mnc']);
   if cobra_param.add_outputs(1),
      copyfile(['phase1ecc____perc_bold_signal.mnc'], ...
               [outfilebase '_ecc_perc_signal.mnc']);
      copyfile(['phase2angle__perc_bold_signal.mnc'], ...
               [outfilebase '_pol_perc_signal.mnc']);
   end;    
end;


%--- error check
if ~isempty(lasterr),
   disp(sprintf('[%s]:Aborting due to error:\n%s',mfilename,lasterr));
   cd(cobra_param.cur_dir);
   if ~cobra_param.keep_tmp,
      delete(cobra_param.ret_tmp_subdir);
   end;
   return;
end;

disp(sprintf('[%s]: Done creating phase-maps.',mfilename));
save(MySaveName);

%-----------------------------------------------------------------------
%--- phase shifting files
%-----------------------------------------------------------------------
files=getfiles([],'*fft_phase.mnc');
[p Files2PhS]=getbase(files);

for n=1:size(Files2PhS,1),
   disp(sprintf('[%s]: PhaseShifting file # %d of %d',...
                mfilename,n,size(Files2PhS,1)));
   PhaseShift(deblank(Files2PhS(n,:)),[],cobra_param.number_phase_shifts);
end;

disp(sprintf('[%s]: Done creating phase-shifted files.',mfilename));
save(MySaveName);

%-----------------------------------------------------------------------
%--- get derivatives non-linear
%-----------------------------------------------------------------------
files=getfiles([],'*_fft_phase*.mnc');
[p PhaseFiles]=getbase(files);

for n=1:size(PhaseFiles,1),
   disp(sprintf('[%s]: Getting derivatives of file # %d of %d',...
                mfilename,n,size(PhaseFiles,1)));
   GetDerivatives2(deblank(PhaseFiles(n,:)),...
                   cobra_param.derivatives_kernel,...
                   cobra_param.add);
end;

disp(sprintf('[%s]: Done creating derivatives.',mfilename));
save(MySaveName);

%-----------------------------------------------------------------------
%--- combining phaseshifted derivatives
%-----------------------------------------------------------------------
files=getfiles([],'phase*dx.mnc');
[p fileDx]=getbase(files);
files=getfiles([],'phase*dy.mnc');
[p fileDy]=getbase(files);
files=getfiles([],'phase*dz.mnc');
[p fileDz]=getbase(files);

for n=1:1+cobra_param.number_phase_shifts:size(fileDx,1)-cobra_param.number_phase_shifts,
   [path,name,base,baseid]=getbase(deblank(...
       fileDx(n+cobra_param.number_phase_shifts,:)));
   base2=base(1,1:(size(base,2)-3));
   base=[base2 '_avgdx.mnc'];
   RetAvg2(fileDx(n:n+cobra_param.number_phase_shifts,:),...
           base,0,1,0,cobra_param.add);
    
   [path,name,base,baseid]=getbase(deblank(...
       fileDy(n+cobra_param.number_phase_shifts,:)));
   base2=base(1,1:(size(base,2)-3));
   base=[base2 '_avgdy.mnc'];
   RetAvg2(fileDy(n:n+cobra_param.number_phase_shifts,:),...
           base,0,1,0,cobra_param.add);

   [path,name,base,baseid]=getbase(deblank(...
       fileDz(n+cobra_param.number_phase_shifts,:)));
   base2=base(1,1:(size(base,2)-3));
   base=[base2 '_avgdz.mnc'];
   RetAvg2(fileDz(n:n+cobra_param.number_phase_shifts,:),...
           base,0,1,0,cobra_param.add);
   
   disp(sprintf('[%s]: Done avg-phs-der set %d of %d', mfilename, ...
        (n+cobra_param.number_phase_shifts)/...
                (1+cobra_param.number_phase_shifts), ...
        size(fileDx,1)/(1+cobra_param.number_phase_shifts)));

end;

disp(sprintf('[%s]: Done combining derivatives.',mfilename));
save(MySaveName);

%--- blurring of derivative files
files=getfiles([],'*avgd*');
[p Der2Blur]=getbase(files);

if cobra_param.smooth_derivatives,
   %--- blurring of derivative files
   if sum(size(cobra_param.derivatives_kernel))==2, % 1 by 1 matrix
      mykernel=sprintf(' %d %d %d ', ...
                       cobra_param.derivatives_kernel,...
                       cobra_param.derivatives_kernel,...
                       cobra_param.derivatives_kernel);
   else, % assume 1 by 3 matrix
      [m,n] = size(cobra_param.derivatives_kernel);
      if m == 1,       % If a row, turn into column vector
         cobra_param.derivatives_kernel = cobra_param.derivatives_kernel(:);
      end;
      mykernel=sprintf(' %d %d %d ', ...
                       cobra_param.derivatives_kernel(1),...
                       cobra_param.derivatives_kernel(2),...
                       cobra_param.derivatives_kernel(3));
   end;

   % choose weighting type
   if cobra_param.weight_derivatives,
      if cobra_param.add_outputs(2),
         system(['mincmath -clamp -const2 0 9999 ' ...
               'phase1ecc____dspear_tmap.mnc ' ...
               'phase1ecc____clp_dspear_tmap.mnc ']);
         system(['mincmath -clamp -const2 0 9999 ' ...
               'phase2angle__dspear_tmap.mnc ' ...
               'phase2angle__clp_dspear_tmap.mnc ']);
         weigth_ext = '_clp_dspear_tmap.mnc';
      else,
         weigth_ext = '_fft_magnitude.mnc';
      end;
   end;
   
   for n=1:size(Der2Blur,1),
      [path,name,base,baseid]=getbase(deblank(Der2Blur(n,:)));
      base2=base(1,1:(size(base,2)-1));
      
      % do weighting first if required
      if cobra_param.weight_derivatives,
         file_to_blur=[base '_tmp.mnc'];
         system(['mincmath -clob -mult ' Der2Blur(n,:) ' ' ...
               base(1,1:12) weigth_ext ' ' ...
               file_to_blur]);
      else,
         file_to_blur=deblank(Der2Blur(n,:));
      end;
      
      if ~cobra_param.get_second_ders,
         system(['mincblur -gaussian -3dfwhm ' ...
               mykernel file_to_blur ' ' base2]);
         copyfile([base2 '_blur.mnc'],[base2 '_blur_d' ...
               base(1,(size(base,2)-1):size(base,2)) '.mnc']);
         delete([base2 '_blur.mnc']);
      else,
         system(['mincblur -gaussian -partial -3dfwhm ' ...
            mykernel file_to_blur ' ' base2]);
         copyfile([base2 '_blur.mnc'],[base2 '_blur_d' ...
             base(1,(size(base,2)-1):size(base,2)) '.mnc']);
         delete([base2 '_blur.mnc']);
         copyfile([base2 '_dxyz.mnc'],[base2 '_2ndxyz_d' ...
             base(1,(size(base,2)-1):size(base,2)) '.mnc']);
         delete([base2 '_dxyz.mnc']);
         % remove unwanted partial derivative files
         delete([base2 '_dx.mnc']);
         delete([base2 '_dy.mnc']);
         delete([base2 '_dz.mnc']);
      end;
      disp(sprintf('[%s]: Done blurring-der %d of %d',...
                   mfilename,n,size(Der2Blur,1)));

      % second cleaning up
      if cobra_param.keep_tmp,
         delete(deblank(Der2Blur(n,:)));
      end;
   end;
   disp(sprintf('[%s]: Done blurring derivatives.',mfilename));
   save(MySaveName);
else,
   for n=1:size(Der2Blur,1),
      [path,name,base,baseid]=getbase(deblank(Der2Blur(n,:)));
      base2=base(1,1:(size(base,2)-1));
      copyfile(deblank(Der2Blur(n,:)),[base2 '_blur_d' ...
           base(1,(size(base,2)-1):size(base,2)) '.mnc']);
   end;
end;


%-----------------------------------------------------------------------
%--- pre-processing of anatomy files
%-----------------------------------------------------------------------
if isempty(cobra_param.invxfm_file),
   if ~isempty(cobra_param.xfm_file),
      cobra_param.invxfm_file=[];
      invxfm_file_base='tmpinvxfmfile';
      
      for n=1:size(xfm_file,1),
         system(['xfminvert ' cobra_param.xfm_file ' ' invxfm_file_base ...
               sprintf('%d',n) '.xfm']);
         cobra_param.invxfm_file=strvcat(cobra_param.invxfm_file,...
                                         [invxfm_file_base ...
                                          sprintf('%d',n) '.xfm']);
      end;
   else,
      cobra_param.invxfm_file=[];
   end;
end;
%--- actual pre-processing
cobra_anatpreprocess(cobra_param);

disp(sprintf('[%s]: Done preprocessing anatomy file.',mfilename));
save(MySaveName);

%-----------------------------------------------------------------------
%--- VFS calculation
%-----------------------------------------------------------------------
calc_vfs2('anatomy_toFMRI_blur_toRet_', ...
          'phase1ecc____fft_magnitude.mnc', ...
          'phase1ecc____fft_phase_avgd_blur_d', ...
          'phase2angle__fft_magnitude.mnc', ...
          'phase2angle__fft_phase_avgd_blur_d', ...
           cobra_param.grad_thresh,...
           cobra_param.fft_mag_thresh, ...
           outfilebase, ...
           cobra_param.get_gradient_map);

disp(sprintf('[%s]: Done VFS calculation.',mfilename));
save(MySaveName);

%--- combining phase and mag maps
% average
MagAvg=[outfilebase '_avg_fft_magnitude.mnc'];
system([' mincaverage phase1ecc____fft_magnitude.mnc ' ...
      ' phase2angle__fft_magnitude.mnc ' MagAvg]);
system(['mincmath -mult ' MagAvg ' ' outfilebase '_VFS.mnc ' ...
       outfilebase '_mVFS.mnc']);
% minimums
%MagAvg=[outfilebase '_min_fft_magnitude.mnc'];
%system([' mincmath -min phase1ecc____fft_magnitude.mnc ' ...
%        ' phase2angle__fft_magnitude.mnc ' MagAvg]);
%system(['mincmath -mult ' MagAvg ' ' outfilebase '_VFS.mnc ' ...
%        outfilebase '_min_mVFS.mnc']);
if cobra_param.get_gradient_map,
   system(['mincmath -min ' outfilebase '_grada.mnc ' ...
           outfilebase '_gradp.mnc ' ...
           outfilebase '_grade.mnc ' ...
           outfilebase '_min_grad.mnc ']);
   system(['mincmath -mult ' outfilebase '_min_mVFS.mnc ' ...
           outfilebase '_min_grad.mnc ' ...
           outfilebase '_min_mgVFS.mnc ']);
end;

         
if cobra_param.add_outputs(2),
   %--- clip tmap to only have pos values
   system(['mincmath -clamp -const2 0 9999 ' ...
           outfilebase '_avg_dspear_tmap.mnc ' ...
           'phase_clp_avg_dspear_tmap.mnc ']);
%   system(['mincmath -clamp -const2 0 9999 ' ...
%           outfilebase '_min_dspear_tmap.mnc ' ...
%           'phase_clp_min_dspear_tmap.mnc ']);
   %--- combining phase and t maps
   system(['mincmath -mult ' outfilebase '_VFS.mnc ' ...
           'phase_clp_avg_dspear_tmap.mnc ' ...
           outfilebase '_tVFS.mnc ']);
%   system(['mincmath -mult ' outfilebase '_VFS.mnc ' ...
%           'phase_clp_min_dspear_tmap.mnc ' ...
%           outfilebase '_min_tVFS.mnc ']);
%   if cobra_param.get_gradient_map,
%      system(['mincmath -mult ' outfilebase '_min_tVFS.mnc ' ...
%              outfilebase '_min_grad.mnc ' ...
%              outfilebase '_min_tgVFS.mnc ']);
%   end;  
end;


%-----------------------------------------------------------------------
%--- clean up
%-----------------------------------------------------------------------
save(MySaveName);

cd(cobra_param.cur_dir);
if ~cobra_param.keep_tmp,
   delfiles=getfiles(cobra_param.ret_tmp_subdir);
   for n=1:size(delfiles,1),
      delete(deblank(delfiles(n,:)));
   end;
   delete(cobra_param.ret_tmp_subdir);
end;

if ~isempty(lasterr),
   disp(sprintf('\nLast error encountered:\n%s',lasterr));
end;

elapsedtime.tmin=round(etime(clock,starttime)/60); %minutes
elapsedcpu.tmin =round((cputime-startcpu)/60);
elapsedtime.hrs =floor(elapsedtime.tmin/60);
elapsedcpu.hrs  =floor(elapsedcpu.tmin /60);
elapsedtime.min =floor(elapsedtime.tmin-(60*elapsedtime.hrs));
elapsedcpu.min  =floor(elapsedcpu.tmin -(60*elapsedcpu.hrs ));

disp(sprintf('[%s]: Finished in %d hours and %d minutes (cpu: %d:%d).',...
             mfilename, ...
             elapsedtime.hrs,elapsedtime.min,...
             elapsedcpu.hrs ,elapsedcpu.min));

%-----------------------------------------------------------------------
%--- end
%-----------------------------------------------------------------------
